@@ -12,29 +12,26 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
+from tests import mock
 from tests import unittest
 import datetime
 import time
 import base64
 import json
-import tempfile
-
-import mock
 
 import botocore.auth
 import botocore.credentials
 from botocore.compat import HTTPHeaders, urlsplit, parse_qs, six
 from botocore.awsrequest import AWSRequest
-from botocore.vendored.requests.models import Request
 
 
 class BaseTestWithFixedDate(unittest.TestCase):
     def setUp(self):
-        self.datetime_patch = mock.patch('botocore.auth.datetime')
-        self.datetime_mock = self.datetime_patch.start()
         self.fixed_date = datetime.datetime(2014, 3, 10, 17, 2, 55, 0)
-        self.datetime_mock.datetime.utcnow.return_value = self.fixed_date
-        self.datetime_mock.datetime.strptime.return_value = self.fixed_date
+        self.datetime_patch = mock.patch('botocore.auth.datetime.datetime')
+        self.datetime_mock = self.datetime_patch.start()
+        self.datetime_mock.utcnow.return_value = self.fixed_date
+        self.datetime_mock.strptime.return_value = self.fixed_date
 
     def tearDown(self):
         self.datetime_patch.stop()
@@ -199,7 +196,7 @@ class TestSigV2(unittest.TestCase):
                      u'VCtWuwaOL0yMffAT8W4y0AFW3W4KUykBqah9S40rB+Q='))
 
     def test_fields(self):
-        request = Request()
+        request = AWSRequest()
         request.url = '/'
         request.method = 'POST'
         request.data = {'Foo': u'\u2713'}
@@ -214,7 +211,7 @@ class TestSigV2(unittest.TestCase):
 
     def test_resign(self):
         # Make sure that resigning after e.g. retries works
-        request = Request()
+        request = AWSRequest()
         request.url = '/'
         request.method = 'POST'
         params = {
@@ -226,6 +223,20 @@ class TestSigV2(unittest.TestCase):
             result, ('Foo=%E2%9C%93',
                      u'VCtWuwaOL0yMffAT8W4y0AFW3W4KUykBqah9S40rB+Q='))
 
+    def test_get(self):
+        request = AWSRequest()
+        request.url = '/'
+        request.method = 'GET'
+        request.params = {'Foo': u'\u2713'}
+        self.signer.add_auth(request)
+        self.assertEqual(request.params['AWSAccessKeyId'], 'foo')
+        self.assertEqual(request.params['Foo'], u'\u2713')
+        self.assertEqual(request.params['Timestamp'], '2014-06-20T08:40:23Z')
+        self.assertEqual(request.params['Signature'],
+                         u'Un97klqZCONP65bA1+Iv4H3AcB2I40I4DBvw5ZERFPw=')
+        self.assertEqual(request.params['SignatureMethod'], 'HmacSHA256')
+        self.assertEqual(request.params['SignatureVersion'], '2')
+
 
 class TestSigV3(unittest.TestCase):
 
@@ -273,13 +284,14 @@ class TestSigV3(unittest.TestCase):
 
 class TestS3SigV4Auth(BaseTestWithFixedDate):
 
+    AuthClass = botocore.auth.S3SigV4Auth
     maxDiff = None
 
     def setUp(self):
         super(TestS3SigV4Auth, self).setUp()
         self.credentials = botocore.credentials.Credentials(
             access_key='foo', secret_key='bar', token='baz')
-        self.auth = botocore.auth.S3SigV4Auth(
+        self.auth = self.AuthClass(
             self.credentials, 'ec2', 'eu-central-1')
         self.request = AWSRequest(data=six.BytesIO(b"foo bar baz"))
         self.request.method = 'PUT'
@@ -307,12 +319,16 @@ class TestS3SigV4Auth(BaseTestWithFixedDate):
         request.method = 'GET'
         credentials = botocore.credentials.Credentials('access_key',
                                                        'secret_key')
-        auth = botocore.auth.S3SigV4Auth(credentials, 's3', 'us-east-1')
+        auth = self.AuthClass(credentials, 's3', 'us-east-1')
         auth.add_auth(request)
         self.assertTrue(
             request.headers['Authorization'].startswith('AWS4-HMAC-SHA256'))
 
     def test_query_string_params_in_urls(self):
+        if not hasattr(self.AuthClass, 'canonical_query_string'):
+            raise unittest.SkipTest('%s does not expose interim steps' %
+                                    self.AuthClass.__name__)
+
         request = AWSRequest()
         request.url = (
             'https://s3.amazonaws.com/bucket?'
@@ -334,7 +350,7 @@ class TestS3SigV4Auth(BaseTestWithFixedDate):
         request.headers[header] = value
         credentials = botocore.credentials.Credentials('access_key',
                                                        'secret_key')
-        auth = botocore.auth.S3SigV4Auth(credentials, 's3', 'us-east-1')
+        auth = self.AuthClass(credentials, 's3', 'us-east-1')
         auth.add_auth(request)
         self.assertNotIn(header, request.headers['Authorization'])
 
@@ -348,20 +364,6 @@ class TestS3SigV4Auth(BaseTestWithFixedDate):
     def test_blacklist_headers(self):
         self._test_blacklist_header('user-agent', 'botocore/1.4.11')
 
-    def test_context_sets_signing_region(self):
-        original_signing_region = 'eu-central-1'
-        new_signing_region = 'us-west-2'
-        self.auth.add_auth(self.request)
-        auth = self.request.headers['Authorization']
-        self.assertIn(original_signing_region, auth)
-        self.assertNotIn(new_signing_region, auth)
-
-        self.request.context = {'signing': {'region': new_signing_region}}
-        self.auth.add_auth(self.request)
-        auth = self.request.headers['Authorization']
-        self.assertIn(new_signing_region, auth)
-        self.assertNotIn(original_signing_region, auth)
-
     def test_uses_sha256_if_config_value_is_true(self):
         self.client_config.s3['payload_signing_enabled'] = True
         self.auth.add_auth(self.request)
@@ -546,17 +548,53 @@ class TestSigV4(unittest.TestCase):
         headers = auth.canonical_headers(original)
         self.assertEqual(headers, 'foo:leading and trailing')
 
+    def test_strips_http_default_port(self):
+        request = AWSRequest()
+        request.url = 'http://s3.us-west-2.amazonaws.com:80/'
+        request.method = 'GET'
+        auth = self.create_signer('s3', 'us-west-2')
+        actual = auth.headers_to_sign(request)['host']
+        expected = 's3.us-west-2.amazonaws.com'
+        self.assertEqual(actual, expected)
+
+    def test_strips_https_default_port(self):
+        request = AWSRequest()
+        request.url = 'https://s3.us-west-2.amazonaws.com:443/'
+        request.method = 'GET'
+        auth = self.create_signer('s3', 'us-west-2')
+        actual = auth.headers_to_sign(request)['host']
+        expected = 's3.us-west-2.amazonaws.com'
+        self.assertEqual(actual, expected)
+
+    def test_strips_http_auth(self):
+        request = AWSRequest()
+        request.url = 'https://username:password@s3.us-west-2.amazonaws.com/'
+        request.method = 'GET'
+        auth = self.create_signer('s3', 'us-west-2')
+        actual = auth.headers_to_sign(request)['host']
+        expected = 's3.us-west-2.amazonaws.com'
+        self.assertEqual(actual, expected)
+
+    def test_strips_default_port_and_http_auth(self):
+        request = AWSRequest()
+        request.url = 'http://username:password@s3.us-west-2.amazonaws.com:80/'
+        request.method = 'GET'
+        auth = self.create_signer('s3', 'us-west-2')
+        actual = auth.headers_to_sign(request)['host']
+        expected = 's3.us-west-2.amazonaws.com'
+        self.assertEqual(actual, expected)
+
 
 class TestSigV4Resign(BaseTestWithFixedDate):
 
     maxDiff = None
+    AuthClass = botocore.auth.SigV4Auth
 
     def setUp(self):
         super(TestSigV4Resign, self).setUp()
         self.credentials = botocore.credentials.Credentials(
             access_key='foo', secret_key='bar', token='baz')
-        self.auth = botocore.auth.SigV4Auth(self.credentials,
-                                            'ec2', 'us-west-2')
+        self.auth = self.AuthClass(self.credentials, 'ec2', 'us-west-2')
         self.request = AWSRequest()
         self.request.method = 'PUT'
         self.request.url = 'https://ec2.amazonaws.com/'
@@ -639,8 +677,8 @@ class TestS3SigV2Presign(BasePresignTest):
         self.assertEqual(query_string['AWSAccessKeyId'], self.access_key)
         self.assertEqual(query_string['Expires'],
                          str(int(self.current_epoch_time) + self.expires))
-        self.assertEquals(query_string['Signature'],
-                          'ZRSgywstwIruKLTLt/Bcrf9H1K4=')
+        self.assertEqual(query_string['Signature'],
+                         'ZRSgywstwIruKLTLt/Bcrf9H1K4=')
 
     def test_presign_with_x_amz_headers(self):
         self.request.headers['x-amz-security-token'] = 'foo'
@@ -649,8 +687,8 @@ class TestS3SigV2Presign(BasePresignTest):
         query_string = self.get_parsed_query_string(self.request)
         self.assertEqual(query_string['x-amz-security-token'], 'foo')
         self.assertEqual(query_string['x-amz-acl'], 'read-only')
-        self.assertEquals(query_string['Signature'],
-                          '5oyMAGiUk1E5Ry2BnFr6cIS3Gus=')
+        self.assertEqual(query_string['Signature'],
+                         '5oyMAGiUk1E5Ry2BnFr6cIS3Gus=')
 
     def test_presign_with_content_headers(self):
         self.request.headers['content-type'] = 'txt'
@@ -659,21 +697,22 @@ class TestS3SigV2Presign(BasePresignTest):
         query_string = self.get_parsed_query_string(self.request)
         self.assertEqual(query_string['content-type'], 'txt')
         self.assertEqual(query_string['content-md5'], 'foo')
-        self.assertEquals(query_string['Signature'],
-                          '/YQRFdQGywXP74WrOx2ET/RUqz8=')
+        self.assertEqual(query_string['Signature'],
+                         '/YQRFdQGywXP74WrOx2ET/RUqz8=')
 
     def test_presign_with_unused_headers(self):
         self.request.headers['user-agent'] = 'botocore'
         self.auth.add_auth(self.request)
         query_string = self.get_parsed_query_string(self.request)
         self.assertNotIn('user-agent', query_string)
-        self.assertEquals(query_string['Signature'],
-                          'ZRSgywstwIruKLTLt/Bcrf9H1K4=')
+        self.assertEqual(query_string['Signature'],
+                         'ZRSgywstwIruKLTLt/Bcrf9H1K4=')
 
 
 class TestSigV4Presign(BasePresignTest):
 
     maxDiff = None
+    AuthClass = botocore.auth.SigV4QueryAuth
 
     def setUp(self):
         self.access_key = 'access_key'
@@ -682,7 +721,7 @@ class TestSigV4Presign(BasePresignTest):
                                                             self.secret_key)
         self.service_name = 'myservice'
         self.region_name = 'myregion'
-        self.auth = botocore.auth.SigV4QueryAuth(
+        self.auth = self.AuthClass(
             self.credentials, self.service_name, self.region_name, expires=60)
         self.datetime_patcher = mock.patch.object(
             botocore.auth.datetime, 'datetime',
@@ -828,6 +867,18 @@ class TestSigV4Presign(BasePresignTest):
         }
         self.assertEqual(query_string, expected_query_string)
 
+    def test_presign_content_type_form_encoded_not_signed(self):
+        request = AWSRequest()
+        request.method = 'GET'
+        request.url = 'https://myservice.us-east-1.amazonaws.com/'
+        request.headers['Content-Type'] = (
+            'application/x-www-form-urlencoded; charset=utf-8'
+        )
+        self.auth.add_auth(request)
+        query_string = self.get_parsed_query_string(request)
+        signed_headers = query_string.get('X-Amz-SignedHeaders')
+        self.assertNotIn('content-type', signed_headers)
+
 
 class BaseS3PresignPostTest(unittest.TestCase):
     def setUp(self):
