@@ -18,8 +18,8 @@ AWS provides a test suite for signature version 4:
 https://github.com/awslabs/aws-c-auth/tree/v0.3.15/tests/aws-sig-v4-test-suite
 
 This module contains logic to run these tests.  The test files were
-placed in ./aws4_testsuite, and we're using nose's test generators to
-dynamically generate testcases based on these files.
+placed in ./aws4_testsuite, and we're using those to dynamically
+generate testcases based on these files.
 
 """
 import os
@@ -28,6 +28,8 @@ import io
 import datetime
 import re
 
+import pytest
+
 from tests import mock
 
 import botocore.auth
@@ -80,13 +82,7 @@ class RawHTTPRequest(six.moves.BaseHTTPServer.BaseHTTPRequestHandler):
         self.error_message = message
 
 
-def test_generator():
-    datetime_patcher = mock.patch.object(
-        botocore.auth.datetime, 'datetime',
-        mock.Mock(wraps=datetime.datetime)
-    )
-    mocked_datetime = datetime_patcher.start()
-    mocked_datetime.utcnow.return_value = DATE
+def generate_test_cases():
     for (dirpath, dirnames, filenames) in os.walk(TESTSUITE_DIR):
         if not any(f.endswith('.req') for f in filenames):
             continue
@@ -96,10 +92,20 @@ def test_generator():
             log.debug("Skipping test: %s", test_case)
             continue
 
-        if HAS_CRT:
-            yield (_test_crt_signature_version_4, test_case)
-        else:
-            yield (_test_signature_version_4, test_case)
+        yield test_case
+
+
+@pytest.mark.parametrize("test_case", generate_test_cases())
+def test_signature_version_4(test_case):
+    datetime_patcher = mock.patch.object(
+        botocore.auth.datetime, 'datetime',
+        mock.Mock(wraps=datetime.datetime)
+    )
+    mocked_datetime = datetime_patcher.start()
+    mocked_datetime.utcnow.return_value = DATE
+
+    _test_signature_version_4(test_case)
+
     datetime_patcher.stop()
 
 
@@ -132,7 +138,7 @@ def create_request_from_raw_request(raw_request):
 
 
 def _test_signature_version_4(test_case):
-    test_case = _SignatureTestCase(test_case)
+    test_case = SignatureTestCase(test_case)
     request = create_request_from_raw_request(test_case.raw_request)
 
     auth = botocore.auth.SigV4Auth(test_case.credentials, SERVICE, REGION)
@@ -156,7 +162,7 @@ def _test_signature_version_4(test_case):
 
 
 def _test_crt_signature_version_4(test_case):
-    test_case = _SignatureTestCase(test_case)
+    test_case = SignatureTestCase(test_case)
     request = create_request_from_raw_request(test_case.raw_request)
 
     # Use CRT logging to diagnose interim steps (canonical request, etc)
@@ -178,7 +184,7 @@ def assert_equal(actual, expected, raw_request, part):
         raise AssertionError(message)
 
 
-class _SignatureTestCase(object):
+class SignatureTestCase(object):
     def __init__(self, test_case):
         filepath = os.path.join(TESTSUITE_DIR, test_case,
                                 os.path.basename(test_case))
